package Org::Ruka::CommandUid;

use warnings;
use strict;

use base qw/Net::IMAP::Server::Command::Uid/;

use Coro;

# same as SUPER, except COPYUID is optional
sub copy {
    warn "in Org::Ruka::CommandUid";
    my $self = shift;
    
    return $self->bad_command("Not enough options") if @_ < 2;
    return $self->bad_command("Too many options") if @_ > 2;

    my ( $messages, $name ) = @_;
    my $mailbox = $self->connection->model->lookup( $name );
    return $self->no_command("[TRYCREATE] Mailbox does not exist") unless $mailbox;
    return $self->bad_command("Mailbox is read-only") if $mailbox->read_only;

    my @messages = $self->connection->selected->get_uids($messages);
    return $self->no_command("Permission denied") if grep {not $_->copy_allowed($mailbox)} @messages;

    my $uidvalidity = $mailbox->uidvalidity;
    my @new;
    for my $m (@messages) {
        my $mprime = $m->copy($mailbox);
        if (defined($mprime)) {
            push @new, $mprime;
        } else {
            $uidvalidity = 'invalid';
        }
        cede;
    }
    my $sequence = join(",",map {$_->uid} @messages);
    my $uids     = join(",",map {$_->uid} @new);
    if ($uidvalidity eq $mailbox->uidvalidity) {
        $self->ok_command("[COPYUID $uidvalidity $sequence $uids] COPY COMPLETED");
    } else {
        $self->ok_command("COPY COMPLETED");
    }
}

1;
