package Org::Ruka::HandleSocket;

use strict;
use warnings;
use Coro;
use AnyEvent::Handle;

our $count = 0;

sub new {
    my ($class) = shift;
    my $self = bless {}, $class;
    $self->{'handle'} = new AnyEvent::Handle(@_, on_error => sub {
        my ($handle, $fatal, $msg) = @_;
        warn "got error $msg"; },
        on_connect => rouse_cb
    );
    
    $self->{'count'} = ++$count;
    my ($handle, $host, $port) = rouse_wait;
    #warn "$self->{count} connected to $host:$port";
    $self->{'opened'} = 1;
    if ($self->{'handle'}) {
        return $self;
    }
    warn "$self->{count} no connection";
    return undef;
}

sub close {
    my $self = shift;
    #warn "$self->{count} closing";
    $self->{'opened'} = 0;
    $self->{'handle'}->push_shutdown();
}

sub write {
    my $self = shift;
    local $_;
    foreach (@_) {
        #warn "$self->{count} write: $_";
        $self->{'handle'}->push_write($_);
    }
}

sub flush {
    # do nothing for now
}

sub read {
    my $self = shift;
    $self->{'handle'}->push_read(chunk => $_[1], rouse_cb);
    my ($handle, $data) = rouse_wait;
    #warn "$self->{count} read $data\n";
    $_[0] = $data;
    return $_[1];
}

sub getline {
    my $self = shift;
    $self->{'handle'}->push_read (line => $/, rouse_cb);
    my ($handle, $line, $eof) = rouse_wait;
    $line .= $eof;
    #warn "$self->{count} readline: $line";
    return $line;
}

sub opened {
    my $self = shift;
    return $self->{'opened'};
}


1;    
    

