package Org::Ruka::ProxyModel;
use strict;
use warnings;
use base qw 'Net::IMAP::Server::DefaultModel';
use Org::Ruka::ProxyMailbox;
use Data::Dumper;

sub init {
    my $self = shift;
    
    foreach (Net::IMAP::Server->concurrent_user_connections) {
        if ($_->auth->imappool == $self->auth->imappool and
            $_->model and $_->model->root) {
            $self->root($_->model->root);
            warn "Reusing root";
            last;
        }
    }
    if (!$self->root) {
        warn "New root";
        $self->root (Org::Ruka::ProxyMailbox->new({'imappool' => $self->auth->imappool}));
        my $separator = $self->auth->imappool->separator;
        my $folders = $self->auth->imappool->folders_more;
        my %tree = ( 'box' => $self->root, 'tree' => {} );
        foreach my $folder (sort keys %$folders) {
            my $parent = \%tree;
            my @parts;
            if ($separator eq '.') {
                @parts = split (/\./, $folder);
            } elsif ($separator eq '/') {
                @parts = split (m@/@, $folder);
            } else {
                die "unknown separator: $separator";
            }
                
            if (@parts == 0) {
                die "could not split folder: $folder";
            }
            
            for (my $i = 0; $i < @parts; ++$i) {
                if (!defined($parent->{'tree'}{$parts[$i]})) {
                    my $box = $parent->{'box'}->add_child('name' => $parts[$i], 'imappool' => $self->auth->imappool, 'listflags' => $folders->{$folder}{'flags'});
                    die "couldn't create $folder" if !$box;
                    $parent->{'tree'}{$parts[$i]} = {'box' => $box, 'tree' => {}};
                }
                $parent = $parent->{'tree'}{$parts[$i]};
            }
        }
    }
    $self->root->inc_refcount();
    return $self;
}

sub DESTROY {
    my $self = shift;
    $self->root->prep_for_destroy();
}
1;
